@script RequireComponent( CharacterController )
@script RequireComponent( PlayerAnimations )

public var fallSpeed : float = 5.0f;
public var movementSpeed : float = 5.0f;
public var dashMultiplier : float = 10.0f;
public var rotationSpeed : float = 1.0f;
public var lookDir : Vector3 = Vector3( -1, -1, 0 );
public var jumpSpeed : float = 2.0f;
public var dashDuration : float = 0.5f;
public var lastCheckpoint : Checkpoint;
public var canDash : boolean = false;

public var dashSound : AudioClip;
public var jumpSound : AudioClip;
public var dieSound : AudioClip;
public var winSound : AudioClip;

public var playerModel : GameObject;
public var groundDetector : GroundDetector;
public var deathEffect : GameObject;
public var dashEffect : GameObject;

private var audioSrc : AudioSource;
private var charController : CharacterController;
private var playerAnims : PlayerAnimations;
private var airYSpeed : float = 0;
private var isDashing : boolean = false;
private var lastDashStartTime : float = -1000;
private var lastMoveDir : Vector3 = Vector3( 1, 0, 0 );

private var isDead : boolean = false;
private var firstCheckpoint : Checkpoint; //hax.
private var lastJumpPlay : float = -1000;

function Start()
{
	charController = GetComponent( CharacterController );
	playerAnims = GetComponent( PlayerAnimations );
	audioSrc = GetComponent( AudioSource );
	
	deathEffect.renderer.enabled = false;
	deathEffect.GetComponent( ParticleEmitter ).emit = false;
	dashEffect.GetComponent( ParticleEmitter ).emit = false;
	
	firstCheckpoint = lastCheckpoint;
}

function Update()
{	
	if( Input.GetKeyDown( KeyCode.Escape ) )
	{
		Application.Quit();
	}
	
	if( isDead )
	{
		return;
	}
	HandleMoveInput();
	
	var theClampedPos = transform.position;
	theClampedPos.z = 0;
	transform.position = theClampedPos;
}

function HandleMoveInput()
{
	var theHInput : float = Input.GetAxisRaw( "Horizontal" );
	
	//this is probably why everything is backwards in movement.  LEAVE IT.
	var theMovementDirection : Vector3 =
		transform.TransformDirection( Vector3( theHInput, 0, 0 ) );
		
	theMovementDirection = theMovementDirection.normalized;
	
	if( theMovementDirection.magnitude > 0 && !isDashing )
	{
		lastMoveDir = theMovementDirection;
	}
	
	var theMovementOffset : Vector3;
	
	if( Input.GetButtonDown( "Dash" ) && canDash )
	{
		audioSrc.clip = dashSound;
		audioSrc.Play();
	}
	
	if( Input.GetButton( "Dash" ) && canDash )
	{
		isDashing = true;
		theMovementOffset = lastMoveDir * movementSpeed * dashMultiplier;
		dashEffect.GetComponent( ParticleEmitter ).emit = true;
	}
	else
	{
		isDashing = false;
		theMovementOffset = theMovementDirection * movementSpeed;
		dashEffect.GetComponent( ParticleEmitter ).emit = false;
	}

	if( !charController.isGrounded || airYSpeed > 0 )
	{
		theMovementOffset.y += airYSpeed;
		playerAnims.isJumping = true;
		
		airYSpeed = Mathf.Lerp( airYSpeed, -fallSpeed, Time.deltaTime * 4 ) ;
	}
	else
	{
		playerAnims.isJumping = false;
	}
	
	if( groundDetector.onGround )
	{
		playerAnims.isJumping = false;
		if( Input.GetButtonDown( "Jump" ) || Input.GetAxisRaw( "Vertical" ) > 0 )
		{
			if( Time.time - lastJumpPlay > 0.2 )
			{
				audioSrc.clip = jumpSound;
				audioSrc.Play();
				lastJumpPlay = Time.time;
			}
			airYSpeed = jumpSpeed;
		}
	}

	theMovementOffset *= Time.deltaTime;
	
	charController.Move( theMovementOffset );

	var theLookDir : Vector3;
	
	if( isDashing )
	{
		playerAnims.isJumping = true;
	}
	else if( Mathf.Abs( theHInput ) > 0.01 )
	{
		if( theHInput > 0 )
		{
			theLookDir = Vector3( -1, 0, 1 );
		}
		else
		{
			theLookDir = Vector3( 1, 0, 1 );
		}
		
		playerModel.transform.rotation = Quaternion.Slerp( playerModel.transform.rotation,
		                                       				Quaternion.LookRotation( theLookDir, Vector3.up ),
		                                       				Time.deltaTime * 10 );
		playerAnims.isRunning = true;
	}
	else
	{
		playerAnims.isRunning = false;
	}
}

function Die()
{
	audioSrc.clip = dieSound;
	audioSrc.Play();
	deathEffect.renderer.enabled = true;
	deathEffect.GetComponent( ParticleEmitter ).emit = true;
	isDead = true;
	yield WaitForSeconds( 2 );
	Respawn();
}

function Respawn()
{
	isDashing = false;
	airYSpeed = 0;
	isDead = false;
	deathEffect.renderer.enabled = false;
	deathEffect.GetComponent( ParticleEmitter ).emit = false;
	groundDetector.onGround = false;

	transform.position = lastCheckpoint.spawnPoint.position;
	lastCheckpoint.Reset();
	//playerModel.renderer.enabled = true;
}

function SetCheckpoint( aCheckpoint : Checkpoint )
{
	lastCheckpoint = aCheckpoint;
}

function IsGrounded() : boolean
{
	return groundDetector.onGround;
}

function Win()
{
	audioSrc.clip = winSound;
	audioSrc.Play();
	
	lastCheckpoint = firstCheckpoint;
	//restart at beginning with dash.
	
	isDead = true;
	yield WaitForSeconds( 3 );
	Respawn();
}