public var sinkAmount : float = 5;
public var sinkColor : Color;
public var sinkSpeed : float = 0.5;
public var resetCheckpoint : Checkpoint;

private var originalPos : Vector3;
private var sunkDist : float = 0;
private var originalColor : Color;
private var isSinking : boolean;
private var isResetting = false;
private var isWaitingForPlayerLanding = false;

function Start()
{
	originalColor = GetComponentInChildren( Renderer ).material.color;
	originalPos = transform.position;
	
	resetCheckpoint.AddResettable( this );
}

function SetColor( aColor : Color )
{
	var theRenderers = GetComponentsInChildren( Renderer );
	
	for( var theRenderer : Renderer in theRenderers )
	{
		theRenderer.material.color = aColor;
	}
}

function Reset()
{
	isResetting = true;
	SetColor( originalColor );
	isSinking = false;
	transform.position = originalPos;
	sunkDist = 0;
	
	//yield WaitForSeconds( 0.1 ); // reset hack.  timing dependent?  think/hope not.
	isResetting = false;	
}

function Update()
{
	if( isSinking && sunkDist < sinkAmount )
	{
		var theSinkDist : float = sinkSpeed * Time.deltaTime;
		sunkDist += theSinkDist;
		var thePos = transform.position;
		thePos.y -= theSinkDist;
		
		transform.position = thePos;
	}
}

function StartSinking()
{
	if( isSinking || isResetting ) //it's a hack to make this depend on isResetting.  Oh well.
	{
		return;
	}
	
	isSinking = true;
	SetColor( sinkColor );
	sunkDist = 0;
}

function OnTriggerEnter( aCollider : Collider )
{
	if( isSinking )
	{
		return;
	}
	
	var thePlayer : PlayerController = aCollider.gameObject.GetComponent( PlayerController );
	
	if( thePlayer  )
	{
		if( thePlayer.IsGrounded() )
		{
			StartSinking();
		}
		else
		{
			isWaitingForPlayerLanding = true;
		}
	}
}

function OnTriggerStay( aCollider : Collider )
{
	if( !isWaitingForPlayerLanding || isSinking )
	{
		return;
	}
	
	var thePlayer : PlayerController = aCollider.gameObject.GetComponent( PlayerController );
	
	if( thePlayer && thePlayer.IsGrounded() )
	{
		StartSinking();
		isWaitingForPlayerLanding = false;
	}
}